@echo off

rem for now we'll just pushd the folder in which make.bat resides. useful for visual studio.
pushd %~dp0

rem Check for the outrun-gcc.
if not defined OUTRUN_GCC_PATH ( 
  rem Backup plan: check whether we can find setupenv.bat ourselves, and use it for the time being.
  if exist "..\setupenv.bat" ( 
    call ..\setupenv.bat
    if errorlevel 1 goto error
  ) else (
    echo OUTRUN_GCC_PATH environment variable not set. Please run setupenv.bat!
    exit /b 1
  )
)

set OUTPUT_PATH=output

if "%1"=="clean" goto clean

set OUTRUN_GCC_ARCH=m68k-elf
set OUTRUN_GCC_PREFIX=%OUTRUN_GCC_ARCH%-
if not exist %OUTPUT_PATH% mkdir %OUTPUT_PATH%

rem compile our files.
echo Assembling...
echo boot.s
%OUTRUN_GCC_PREFIX%as boot.s -m68000 --bitwise-or -o %OUTPUT_PATH%\boot.o
if ERRORLEVEL 1 goto error

if exist boot-sub.s ( 
  echo boot-sub.s
  %OUTRUN_GCC_PREFIX%as boot-sub.s -m68000 -o %OUTPUT_PATH%\boot-sub.o
  if ERRORLEVEL 1 goto error
)

echo Linking...
%OUTRUN_GCC_PREFIX%ld %OUTPUT_PATH%\boot.o --script=boot.ld -o %OUTPUT_PATH%\bootloader.bin --Map=%OUTPUT_PATH%\bootloader.map
if ERRORLEVEL 1 goto error

if exist %OUTPUT_PATH%\boot-sub.o (
  %OUTRUN_GCC_PREFIX%ld %OUTPUT_PATH%\boot-sub.o --script=boot-sub.ld -o %OUTPUT_PATH%\bootloader-sub.bin --Map=%OUTPUT_PATH%\bootloader-sub.map
  if ERRORLEVEL 1 goto error
)

rem build rom images.
echo Building ROM images...

%OUTRUN_GCC_PREFIX%objcopy %OUTPUT_PATH%\bootloader.bin %OUTPUT_PATH%\epr-10380b.133 --input-target binary -i 2 --byte 0 --pad-to 0x10000
if ERRORLEVEL 1 goto error
%OUTRUN_GCC_PREFIX%objcopy %OUTPUT_PATH%\bootloader.bin %OUTPUT_PATH%\epr-10382b.118 --input-target binary -i 2 --byte 1 --pad-to 0x10000
if ERRORLEVEL 1 goto error

if exist "%OUTPUT_PATH%\bootloader-sub.bin" (
  %OUTRUN_GCC_PREFIX%objcopy %OUTPUT_PATH%\bootloader-sub.bin %OUTPUT_PATH%\epr-10327a.76 --input-target binary -i 2 --byte 0 --pad-to 0x10000
  if ERRORLEVEL 1 goto error
  %OUTRUN_GCC_PREFIX%objcopy %OUTPUT_PATH%\bootloader-sub.bin %OUTPUT_PATH%\epr-10329a.58 --input-target binary -i 2 --byte 1 --pad-to 0x10000
  if ERRORLEVEL 1 goto error
)

echo Done.
goto end

:clean
set "outputfiles=boot.o bootloader.bin bootloader.map boot-sub.o bootloader-sub.bin bootloader-sub.map epr-10380b.133 epr-10382b.118 epr-10327a.76 epr-10329a.58"
for %%i in (%outputfiles%) do (
  if exist "%OUTPUT_PATH%\%%i" del "%OUTPUT_PATH%\%%i"
)
goto end

:error
echo Build aborted.
echo.
exit /b 1

:end

popd
