#!/bin/bash 

# This makes sure we exit if anything unexpected happens.
set -e

# Check for the outrun-gcc.
if [ -z ${OUTRUN_GCC_PATH} ]; then
 
  # Backup plan: check whether we can find setupenv.sh
  DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
  if [ -f "$DIR/../setupenv.sh" ]; then 
    source $DIR/../setupenv.sh
  else
    echo OUTRUN_GCC_PATH environment variable not set!
    exit 1
  fi

fi

OUTPUT_PATH=output

# quick clean and exit.
if [ "$1" == "clean" ]; then
  declare -a outputfiles=("boot.o" "bootloader.bin" "bootloader.map" "boot-sub.o" "bootloader-sub.bin" "bootloader-sub.map" "epr-10380b.133" "epr-10382b.118" "epr-10327a.76" "epr-10329a.58" )
  for i in "${outputfiles[@]}"
  do
    if [ -f "$OUTPUT_PATH/$i" ]; then
      rm "$OUTPUT_PATH/$i"
    fi
  done
  exit 0
fi

export OUTRUN_GCC_ARCH=m68k-elf
export OUTRUN_GCC_PREFIX=$OUTRUN_GCC_ARCH-
if [ ! -d "$OUTPUT_PATH" ]; then
  mkdir $OUTPUT_PATH
fi

# compile our files.
echo Assembling...
echo boot.s
${OUTRUN_GCC_PREFIX}as boot.s -m68000 --bitwise-or -o "$OUTPUT_PATH/boot.o"

if [ -f "boot-sub.s" ]; then 
  echo boot-sub.s
  ${OUTRUN_GCC_PREFIX}as boot-sub.s -m68000 -o $OUTPUT_PATH/boot-sub.o
fi

echo Linking...
${OUTRUN_GCC_PREFIX}ld $OUTPUT_PATH/boot.o --script=boot.ld -o $OUTPUT_PATH/bootloader.bin --Map=$OUTPUT_PATH/bootloader.map

if [ -f "$OUTPUT_PATH/boot-sub.o" ]; then
  ${OUTRUN_GCC_PREFIX}ld $OUTPUT_PATH/boot-sub.o --script=boot-sub.ld -o $OUTPUT_PATH/bootloader-sub.bin --Map=$OUTPUT_PATH/bootloader-sub.map
fi

# build rom images.
echo Building ROM images...

${OUTRUN_GCC_PREFIX}objcopy $OUTPUT_PATH/bootloader.bin $OUTPUT_PATH/epr-10380b.133 --input-target binary -i 2 --byte 0 --pad-to 0x10000
${OUTRUN_GCC_PREFIX}objcopy $OUTPUT_PATH/bootloader.bin $OUTPUT_PATH/epr-10382b.118 --input-target binary -i 2 --byte 1 --pad-to 0x10000

if [ -f "$OUTPUT_PATH/bootloader-sub.bin" ]; then
  ${OUTRUN_GCC_PREFIX}objcopy $OUTPUT_PATH/bootloader-sub.bin $OUTPUT_PATH/epr-10327a.76 --input-target binary -i 2 --byte 0 --pad-to 0x10000
  ${OUTRUN_GCC_PREFIX}objcopy $OUTPUT_PATH/bootloader-sub.bin $OUTPUT_PATH/epr-10329a.58 --input-target binary -i 2 --byte 1 --pad-to 0x10000
fi

echo Done.

