#include <Platform.h>
#include "lpt.h"
#include "inpout32.h"
#pragma comment(lib, "inpout32.lib")

// Register offsets.
enum
{
	REG_DATA    = 0,
	REG_STATUS  = 1,
	REG_CONTROL = 2,
};

// Base port.
uint32 s_portBase = 0x378;

void LPT_SetBasePort (uint32 port)
{
	s_portBase = port;
}

void LPT_SetData (uint8 dataBits)
{
	Out32 (s_portBase+REG_DATA, (short)dataBits);
}

void LPT_SetControl (uint8 controlBits)
{
	Out32 (s_portBase+REG_CONTROL, (controlBits & 0xf));
}

uint8 LPT_GetControl ()
{
	return (Inp32 (s_portBase+REG_CONTROL) & 0xf);
}

uint8 LPT_GetStatus ()
{
	return ((uint8)Inp32(s_portBase+REG_STATUS)) & 0xF8; // Ignore the 3 unused bits.
}

bool LPT_DriverInstalled ()
{
	return IsInpOutDriverOpen() != FALSE;
}

//=================================================================================================================

// https://docs.microsoft.com/en-us/windows/win32/wmisdk/example--getting-wmi-data-from-the-local-computer
// https://www.downtowndougbrown.com/2014/11/detecting-parallel-ports-and-their-io-addresses-in-windows/

// Not particularly pretty.

#define _WIN32_DCOM
using namespace std;
#include <comdef.h>
#include <Wbemidl.h>
#include <vector>
#pragma comment(lib, "wbemuuid.lib")

static bool InitWMI (IWbemServices*& pSvc, IWbemLocator*& pLoc)
{
    // Step 1: --------------------------------------------------
    // Initialize COM. ------------------------------------------

    HRESULT hres = CoInitializeEx(0, COINIT_MULTITHREADED); 
    if (FAILED (hres))
        return false;

    // Step 2: --------------------------------------------------
    // Set general COM security levels --------------------------

    hres =  CoInitializeSecurity(
        NULL, 
        -1,                          // COM authentication
        NULL,                        // Authentication services
        NULL,                        // Reserved
        RPC_C_AUTHN_LEVEL_DEFAULT,   // Default authentication 
        RPC_C_IMP_LEVEL_IMPERSONATE, // Default Impersonation  
        NULL,                        // Authentication info
        EOAC_NONE,                   // Additional capabilities 
        NULL                         // Reserved
        );
                      
    if (FAILED (hres))
    {
        //cout << "Failed to initialize security. Error code = 0x" 
        //    << hex << hres << endl;
        CoUninitialize();
        return false;
    }
    
    // Step 3: ---------------------------------------------------
    // Obtain the initial locator to WMI -------------------------

    pLoc = NULL;

    hres = CoCreateInstance(
        CLSID_WbemLocator,             
        0, 
        CLSCTX_INPROC_SERVER, 
        IID_IWbemLocator, (LPVOID *) &pLoc);
 
    if (FAILED (hres))
    {
        //cout << "Failed to create IWbemLocator object."
        //    << " Err code = 0x"
        //    << hex << hres << endl;
        CoUninitialize();
        return false;
    }

    // Step 4: -----------------------------------------------------
    // Connect to WMI through the IWbemLocator::ConnectServer method

    pSvc = NULL;
 
    // Connect to the root\cimv2 namespace with
    // the current user and obtain pointer pSvc
    // to make IWbemServices calls.
    hres = pLoc->ConnectServer(
         _bstr_t(L"ROOT\\CIMV2"), // Object path of WMI namespace
         NULL,                    // User name. NULL = current user
         NULL,                    // User password. NULL = current
         0,                       // Locale. NULL indicates current
         NULL,                    // Security flags.
         0,                       // Authority (for example, Kerberos)
         0,                       // Context object 
         &pSvc                    // pointer to IWbemServices proxy
         );
    
    if (FAILED (hres))
    {
        //cout << "Could not connect. Error code = 0x" 
        //     << hex << hres << endl;
        pLoc->Release();     
        CoUninitialize();
        return false;
    }

    //cout << "Connected to ROOT\\CIMV2 WMI namespace" << endl;


    // Step 5: --------------------------------------------------
    // Set security levels on the proxy -------------------------

    hres = CoSetProxyBlanket(
       pSvc,                        // Indicates the proxy to set
       RPC_C_AUTHN_WINNT,           // RPC_C_AUTHN_xxx
       RPC_C_AUTHZ_NONE,            // RPC_C_AUTHZ_xxx
       NULL,                        // Server principal name 
       RPC_C_AUTHN_LEVEL_CALL,      // RPC_C_AUTHN_LEVEL_xxx 
       RPC_C_IMP_LEVEL_IMPERSONATE, // RPC_C_IMP_LEVEL_xxx
       NULL,                        // client identity
       EOAC_NONE                    // proxy capabilities 
    );

    if (FAILED(hres))
    {
        //cout << "Could not set proxy blanket. Error code = 0x" 
        //    << hex << hres << endl;
        pSvc->Release();
        pLoc->Release();     
        CoUninitialize();
        return false;
    }

	return true;
}

static void ShutdownWMI (IWbemServices* pSvc, IWbemLocator* pLoc)
{
    pSvc->Release();
    pLoc->Release();
    CoUninitialize();
}

static bool GetBasePort (OUT uint16& outPort, IWbemServices* pSvc, const wchar_t* devId)
{
	IEnumWbemClassObject* pPortEnumerator = NULL;

	// Not sanitized!
	// One could market a device with injection here, specifically to break orboot.exe!
	int basePort = -1;
	const size_t devLen = wcslen (devId);
	wstring devId2;
	devId2.reserve (devLen);
	for (size_t i=0; i<devLen; i++)
	{
		wchar_t c = devId[i];
		devId2.push_back(c);
		if (c == L'\\') // Double backslashes
			devId2.push_back(c);
	}

	wstring query = wstring (L"ASSOCIATORS OF {Win32_PnPEntity.DeviceID=\"");
	query.append (devId2);
	query.append (L"\"} WHERE ResultClass = Win32_PortResource");

	HRESULT hres = pSvc->ExecQuery(
		bstr_t("WQL"),
		bstr_t(query.c_str()),
		WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY, 
		NULL,
		&pPortEnumerator);
	
	if (SUCCEEDED (hres))
	{
		bool basBasePort = false;
		IWbemClassObject* pclsObj2 = NULL;
		ULONG uReturn2 = 0;
		while (pPortEnumerator)
		{
			HRESULT hr = pPortEnumerator->Next (WBEM_INFINITE, 1, 
				&pclsObj2, &uReturn2);

			if(0 == uReturn2)
				break;

			// We may have multiple ranges here. Take the first, for now.
			VARIANT vtStartAddr;
			hr = pclsObj2->Get (L"StartingAddress", 0, &vtStartAddr, 0, 0);
			if (SUCCEEDED (hr))
			{
				if (vtStartAddr.vt == VT_BSTR)
				{
					unsigned int port;
					if (swscanf (vtStartAddr.bstrVal, L"%u", &port) == 1 && port < 65536)
						basePort = port;
				}
				VariantClear (&vtStartAddr);
			}

			pclsObj2->Release();

			if (basePort != -1)
				break; // One is enough, hopefully the correct one.
		}

		pPortEnumerator->Release ();
	}

	if (basePort != -1)
		outPort = (uint16)basePort;
	return basePort != -1;
}


size_t LPT_DetectPorts (OUT LPT_PORT_INFO** ppInfo)
{
	*ppInfo = NULL;
	vector<LPT_PORT_INFO> lptList; // Copied later on.

	IWbemServices* pSvc;
	IWbemLocator* pLoc;
	if (!InitWMI(pSvc, pLoc))
		return 0;

    // Step 6: --------------------------------------------------
    // Use the IWbemServices pointer to make requests of WMI ----

    // This will detect ports (LPT and COM)
    IEnumWbemClassObject* pEnumerator = NULL;
    HRESULT hres = pSvc->ExecQuery(
        bstr_t("WQL"), 
		bstr_t("SELECT * FROM Win32_PnPEntity WHERE ClassGUID=\"{4D36E978-E325-11CE-BFC1-08002BE10318}\""),
        WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY, 
        NULL,
        &pEnumerator);

    if (FAILED (hres))
    {
		ShutdownWMI(pSvc, pLoc);
        return 0;
    }

    // Step 7: -------------------------------------------------
    // Get the data from the query in step 6 -------------------
 
    IWbemClassObject* pclsObj = NULL;
    ULONG uReturn = 0;
   
	// Walk ports.
    while (pEnumerator)
    {
        HRESULT hr = pEnumerator->Next (WBEM_INFINITE, 1, 
            &pclsObj, &uReturn);

        if(0 == uReturn)
            break;

        VARIANT vtName;

        // Get the value of the Name property
        hr = pclsObj->Get (L"Name", 0, &vtName, 0, 0);
		if (SUCCEEDED (hr) && vtName.vt == VT_BSTR)
		{
			// Filter on 'LPT' here... ugh.
			const wchar_t* lptMatch = wcsstr (vtName.bstrVal, L"(LPT");
			if (lptMatch)
			{
				// This looks like a parallel port.
				LPT_PORT_INFO lptInfoTmp = { 0 };
				lptList.push_back (lptInfoTmp);
				LPT_PORT_INFO& lptInfo = lptList.back();

				const wchar_t* lptEnd = wcschr (lptMatch, ')');
				if (lptEnd)
				{
					size_t nameLen = lptEnd-(lptMatch+1);
					lptInfo.name = (wchar_t*)malloc(sizeof(wchar_t) * (nameLen+1));
					memcpy ((wchar_t*)lptInfo.name, lptMatch+1, nameLen*sizeof(wchar_t));
					((wchar_t*)lptInfo.name)[nameLen] = L'\0';
				}

				VARIANT vtDevId;
				hr = pclsObj->Get (L"DeviceID", 0, &vtDevId, 0, 0);
				if (SUCCEEDED (hr)  && vtDevId.vt == VT_BSTR)
				{
					// Got a device ID, eg:
					// "MF\PCI#VEN_1C00&DEV_3250&SUBSYS_32501C00&REV_10\5&63F317F&0&00A9#CHILD0000"
					size_t devLen = wcslen (vtDevId.bstrVal);
					lptInfo.deviceId = (wchar_t*)malloc(sizeof(wchar_t) * (devLen+1));
					wcscpy ((wchar_t*)lptInfo.deviceId, vtDevId.bstrVal);

					uint16 basePort;
					bool gotPort = GetBasePort (basePort, pSvc, lptInfo.deviceId);
					if (gotPort)
						lptInfo.portAddr = basePort;

					VariantClear (&vtDevId);
				}
			}

	        VariantClear (&vtName);
		}

        pclsObj->Release();
    }

    // Cleanup
    // ========
    
    pEnumerator->Release();

	ShutdownWMI(pSvc, pLoc);

	size_t numResults = lptList.size();
	size_t resultSize = lptList.size() * sizeof(LPT_PORT_INFO);
	LPT_PORT_INFO* result = (LPT_PORT_INFO*)malloc(resultSize);
	memcpy (result, &lptList.front(), resultSize);
	lptList.clear();
	*ppInfo = result;
	return numResults;
}

void LPT_FreePortInfo (LPT_PORT_INFO* lptInfo, size_t count)
{
	if (lptInfo)
	{
		LPT_PORT_INFO* entry = lptInfo;
		for (size_t i=0; i<count; i++)
		{
			if (entry->name)
				free ((wchar_t*)entry->name);
			if (entry->deviceId)
				free ((wchar_t*)entry->deviceId);
		}

		free (lptInfo);
	}
}
