#ifndef __OPTIONPARSE_H__
#define __OPTIONPARSE_H__

// Callback to invoke when an option is found on the command line.
// Return false to abort.
typedef bool OPTIONCALLBACK(const char* value, void* context);

enum OptionFlags
{
	OPT_PARAM = 1,
	OPT_ONCE  = 2,
};

// This describes a single option.
struct Option
{
	const char*     name;	  // Option name (case insensitive).
	uint32          flags;    // Needs an additional parameter.
	OPTIONCALLBACK* callback; // Callback to run.
	void*           context;  // Custom parameter.
};

// Used for non-option arguments.
typedef bool ARGCALLBACK (const char* arg, void* context);

// Parses the command line. Options will be handled by their respective callbacks. Non-options will be handled by pgnArg.
// Returns the number of non-option arguments processed, or -1 if one of the callbacks returned false.
int ParseOptions (int argc, const char* const* argv, ARGCALLBACK* pfnArg, void* context, const Option* pOptions, size_t optionCount);

#endif // __OPTIONPARSE_H__
